"use strict";
/**
 * @author Olsi Rrjolli
 * In these class are all decorators defined that we will use for the Controllers
 */
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../constants/constants");
const reflectMetadataHelpers_1 = require("../utils/reflectMetadataHelpers");
/**
 * HTTP Controller Marker
 * @param resourcePath
 *  The path to the resource for default
 */
/* tslint:disable:variable-name */
exports.Controller = (resourcePath = '') => {
    return (target) => {
        Reflect.defineMetadata(constants_1.controllerMetaKey.RESOURCE_KEY, resourcePath, target);
        const resource = Reflect.getOwnMetadata(constants_1.controllerMetaKey.RESOURCE_KEY, target);
        const metaDataKeys = Reflect.getOwnMetadata(constants_1.controllerMetaKey.METHODS_KEY, target);
        metaDataKeys.forEach((metaData) => {
            Reflect.defineMetadata(reflectMetadataHelpers_1.generateResourceMetaKey(metaData.http, resource, metaData.path), metaData.method, target);
        });
    };
};
/**
 * HTTP POST Marker
 * @param path
 *  The path to the resource for default
 */
exports.POST = (path = '') => {
    return (target, propertyKey, descriptor) => {
        const ownMethods = Reflect.getOwnMetadata(constants_1.controllerMetaKey.METHODS_KEY, target.constructor);
        ownMethods.push({ http: constants_1.httpMethod.POST, path: path, method: propertyKey });
        Reflect.defineMetadata(constants_1.controllerMetaKey.METHODS_KEY, ownMethods, target.constructor);
    };
};
/**
 * HTTP GET Marker, Decorate the method with the GET Decorator
 * @param path
 *  The path to the resource for default
 */
exports.GET = (path = '') => {
    return (target, propertyKey, descriptor) => {
        const ownMethods = Reflect.getOwnMetadata(constants_1.controllerMetaKey.METHODS_KEY, target.constructor) || [];
        ownMethods.push({ http: constants_1.httpMethod.GET, path: path, method: propertyKey });
        Reflect.defineMetadata(constants_1.controllerMetaKey.METHODS_KEY, ownMethods, target.constructor);
    };
};
/**
 * HTTP DELETE Marker
 * @param path
 *  The path to the resource for default
 */
exports.DELETE = (path = '') => {
    return (target, propertyKey, descriptor) => {
        const ownMethods = Reflect.getOwnMetadata(constants_1.controllerMetaKey.METHODS_KEY, target.constructor);
        ownMethods.push({ http: constants_1.httpMethod.DELETE, path: path, method: propertyKey });
        Reflect.defineMetadata(constants_1.controllerMetaKey.METHODS_KEY, ownMethods, target.constructor);
    };
};
/**
 * HTTP PUT Marker
 * @param path
 *  The path to the resource for default
 */
exports.PUT = (path = '') => {
    return (target, propertyKey, descriptor) => {
        const ownMethods = Reflect.getOwnMetadata(constants_1.controllerMetaKey.METHODS_KEY, target.constructor);
        ownMethods.push({ http: constants_1.httpMethod.PUT, path: path, method: propertyKey });
        Reflect.defineMetadata(constants_1.controllerMetaKey.METHODS_KEY, ownMethods, target.constructor);
    };
};
/**
 * PathParam Setter/Marker
 * @param pathParam
 *  The path parameter from the Request
 */
exports.PathParam = (pathParam) => {
    return function (target, propertyKey, parameterIndex) {
        const ownDecorators = Reflect.getOwnMetadata(propertyKey, target, propertyKey) || [];
        ownDecorators.push({ paramName: pathParam, index: parameterIndex });
        Reflect.defineMetadata(propertyKey, ownDecorators, target, propertyKey);
    };
};
/**
 * Query Parameter Setter/Marker
 * @param queryParam
 *  The  query parameter from the Request
 */
exports.QueryParam = (queryParam) => {
    return function (target, propertyKey, parameterIndex) {
        const ownDecorators = Reflect.getOwnMetadata(propertyKey, target, propertyKey) || [];
        ownDecorators.push({ queryName: queryParam, index: parameterIndex });
        Reflect.defineMetadata(propertyKey, ownDecorators, target, propertyKey);
    };
};
/**
 * Mapper Decorator
 * @param type
 *    Decorate the Controller with the Model, which should use for the Mapper
 */
exports.Model = (model) => {
    return (target) => {
        Reflect.defineMetadata(constants_1.controllerMetaKey.MODEL_KEY, model, target);
    };
};
//# sourceMappingURL=ControllerDecorators.js.map